#include <stdio.h>
#include <sys/types.h>
#include <assert.h>
#include <sys/param.h>
#include <stdlib.h>

#include <tcl.h>

#include <process.h>
#include <getdir.h>


#define MAXDEVS 4350


static int initDev[MAXDEVS];
static Tcl_HashTable fid2dirTab[MAXDEVS];

static char *dirDate;
static char *dirDir;

typedef struct {
  FIDType dirid;
  FIDType parentFid;
} FileEntry;


static void Dir_InitDevice(DevType dev);

     
void
Dir_Init(char *dir, char *date)
{
  int i;

  dirDir = dir;
  dirDate = date;
  
  for (i=0; i < MAXDEVS; i++) {
    initDev[i] = 0;
  }
}


/*
 *-----------------------------------------------------------------
 * initDevice --
 *   read the directory group mappings for the given device
 *
 * values are read from a file named dir.date.dev
 * each line in the file has the format:
 * dirid fid parent-fid
 *-----------------------------------------------------------------
 */
static void
Dir_InitDevice(DevType dev)
{
  char inputfilename[MAXPATHLEN];
  FILE *infilep;
  int new;
  FIDType fid, parentFid, dirid;
  Tcl_HashEntry *entryp;
  FileEntry *filep;

  sprintf(inputfilename, "%s/dir.%s.%ld",
	  dirDir, dirDate, dev);

#ifdef DEBUG
  fprintf(stderr, "reading from %s\n", inputfilename);
#endif DEBUG
  
  infilep = fopen(inputfilename, "r");

  if (infilep == NULL) {
    perror("open");
    fprintf(stderr, "error opening %s\n", inputfilename);
    exit(1);
  }

  Tcl_InitHashTable(fid2dirTab + dev, TCL_ONE_WORD_KEYS);
  
  while (fscanf(infilep, "%ld %ld %ld", &dirid, &fid, &parentFid) == 3) {
    entryp = Tcl_CreateHashEntry(fid2dirTab + dev, (char *) fid, &new);

    if (!new) {
      fprintf(stderr, "fid %ld repeated in dir listing for dev %ld\n",
	fid, dev);
      exit(1);
    }
    filep = (FileEntry *) malloc(sizeof(FileEntry));
    assert(filep != NULL);
    filep->parentFid = parentFid;
    filep->dirid = dirid;
    Tcl_SetHashValue (entryp, (char *) filep);
  }
  fclose(infilep);
}


void
Dir_GetParent(DevType dev, FIDType fid, FIDType *parentFid)
{
  Tcl_HashEntry *entryp;
  FileEntry *filep;

  assert ((dev > 0) && (dev < MAXDEVS));
  
  if (initDev[dev] == 0) {
    Dir_InitDevice(dev);
    initDev[dev] = 1;
  }

  entryp = Tcl_FindHashEntry(fid2dirTab + dev, (char *) fid);

  if (entryp == NULL) {
/*    fprintf(stderr, "NO FILE ENTRY: dev %d fid %ld\n", dev, fid);*/
    *parentFid = UNKNOWN;
    return;
  }

  filep = Tcl_GetHashValue(entryp);
  assert(filep);

  *parentFid = filep->parentFid;
  
#ifdef DEBUG
  fprintf(stderr, "found parent: dev %ld fid %ld parentFid %ld\n",
	  dev, fid, *parentFid);
#endif DEBUG
}


void
Dir_GetDirid(DevType dev, FIDType fid, FIDType *dirid)
{
  Tcl_HashEntry *entryp;
  FileEntry *filep;

  assert ((dev > 0) && (dev < MAXDEVS));
  
  if (initDev[dev] == 0) {
    Dir_InitDevice(dev);
    initDev[dev] = 1;
  }

  entryp = Tcl_FindHashEntry(fid2dirTab + dev, (char *) fid);

  if (entryp == NULL) {
/*    fprintf(stderr, "NO FILE ENTRY: dev %d fid %ld\n", dev, fid);*/
    *dirid = UNKNOWN;
    return;
  }

  filep = Tcl_GetHashValue(entryp);
  assert(filep);

  *dirid = filep->dirid;
  
#ifdef DEBUG
  fprintf(stderr, "found group: dev %d fid %ld group %d\n", dev, fid, *dirid);
#endif DEBUG

}
