/*
 * Copyright 1996 Regents of the University of California
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 */
/*
 *-----------------------------------------------------------------
 * post2ascii --
 *   this program reads binary trace data from stdin and
 *   converts it to ascii on stdout.
 *
 *-----------------------------------------------------------------
 */


#include <stdio.h>
#include <sys/types.h>

#include "filter.h"
#include "process.h"



void
processRecords()
{
  char record[MAX_TRACE_RECORD_SIZE];
  newHeader *hp = (newHeader *) record;
  char *args = record + sizeof(newHeader);
  DevFid *devfidp;
  TruncArgs *truncArgs;
  CloseArgs *closeArgs;
  ReadWriteArgs *rwArgs;
  LockfArgs *lockArgs;
  ForkArgs *forkArgs;
  OpenArgs *openArgs;

  /* print heading */
  fprintf(stdout, "%s %10s %7s %6s %6s %6s\n",
	  "TIME", "HOST", "UID", "PID", "CALL", "ARGS");


  
  while (Filter_NextRecord(record) != FILTER_EOF) {

    /* print header information */
    printf("%ld %3lu %8ld %8ld %3hu\t",
	   hp->ah_time, hp->ah_host, hp->ah_uid, hp->ah_pid, hp->ah_event);

    /* print arguments */
    switch (hp->ah_event) {


    case EXIT: case SYNC: case REBOOT: case LSYNC: {
      printf("\n");
      break;
    }


    case MUNMAP: case FSTAT: case FSYNC: case FCHDIR: case FCHOWN:
    case FCHMOD: case GETDIRENTRIES: case FGETACL: case FSETACL:
    case MMAP: case TSYNC: case SMOUNT: case UNLINK: case EXECV:
    case CHDIR: case CHMOD: case CHOWN: case UMOUNT: case UTIME:
    case ACCESS: case STAT: case LSTAT: case RDLINK: case VFSMOUNT:
    case EXECVE: case CHROOT: case MKDIR: case RMDIR: case GETACL:
    case SETACL: case GETACCESS: case LINK: case RENAME: case SYMLINK: {
      devfidp = (DevFid *) args;
      printf("%ld %ld\n", devfidp->dev, devfidp->fid);
      break;
    }


    case TRUNC: case FTRUNC: {
      truncArgs = (TruncArgs *) args;
      printf("%ld %ld %d\n",
	     truncArgs->devfid.dev, truncArgs->devfid.fid, truncArgs->size);
      break;
    }


    case CLOSE: {
      closeArgs = (CloseArgs *) args;
      printf("%ld %ld %d\n",
	     closeArgs->devfid.dev, closeArgs->devfid.fid, closeArgs->closemode);
      break;
    }

    case SREAD: case READV: case SWRITE: case WRITEV: {
      rwArgs = (ReadWriteArgs *) args;
      printf("%ld %ld %d %d\n",
	     rwArgs->devfid.dev, rwArgs->devfid.fid,
	     rwArgs->offset, rwArgs->bytes);
      break;
    }


    case LOCKF: {
      lockArgs = (LockfArgs *) args;
      printf("%ld %ld %d %d\n",
	     lockArgs->devfid.dev, lockArgs->devfid.fid,
	     lockArgs->lockf_function, lockArgs->size);
      break;
    }

    case FORK: case VFORK: {
      forkArgs = (ForkArgs *) args;
      printf("%d %d\n", forkArgs->pid, forkArgs->childFlag);
      break;
    }

    case OPEN: case CREAT: {
      openArgs = (OpenArgs *) args;
      printf("%ld %ld %d %d %d %d %d %d %d %d %d %d\n",
	     openArgs->devfid.dev, openArgs->devfid.fid,
	     openArgs->fd, openArgs->ftype, openArgs->fstype,
	     openArgs->uid, openArgs->size, openArgs->nlink,
	     openArgs->ctime, openArgs->mtime, openArgs->atime,
	     openArgs->mode);
      break;
    }


    case LSEEK: case DUP: case DUP2: case FCNTL: case FSCTL: {
      fprintf(stderr, "ERROR: this call should not be in processed traces.\n");
      exit(1);
    }

    default: {
      fprintf(stderr, "ERROR: unknown call %d, len %d\n",
	      hp->ah_event, hp->ah_len);
      exit(1);
    }
    }
  }
}


int
main(int argc, char **argv)
{
  Filter_Init(0, 0, NULL, 0, NULL);

  processRecords();
  fflush(stdout);

  exit(0);
}
    
